require "active_job"
require "workspace"
require "workspace-archive"
require "workspace-parse"
require "workspace-media"
require "workspace-net"
require 'dronejob/modules/attr_store'
require 'dronejob/modules/callbacks'
require 'dronejob/modules/core'
require 'dronejob/modules/git'
require 'dronejob/modules/log'
require 'dronejob/modules/options'
require 'dronejob/modules/params'
require 'dronejob/modules/phases'
require 'dronejob/modules/workspace'

module Dronejob
  class Base < ActiveJob::Base
    DRONEJOB_TYPE = "base"
    include Modules::AttrStore
    include Modules::Callbacks
    include Modules::Core
    include Modules::Git
    include Modules::Options
    include Modules::Params
    include Modules::Phases
    include Modules::Workspace
    include Modules::Log

    def uuid
      @uuid ||= "#{self.class.name.split('::').last.underscore}_#{SecureRandom.uuid}"
    end

    def title
      self.class.name
    end

    rescue_from(Exception) do |exception|
      error_rescue(exception) if respond_to?(:error_rescue)
      error(exception)
    end

    ActiveSupport.run_load_hooks(:dronejob, self)
  end
end
