require "dronejob"
require "active_job"

module Dronejob
  class Command < Thor
    Dronejob::Loader.workers(:base).each do |identifier, worker|
      desc identifier, "Run #{identifier} worker"

      # Worker options
      method_option(:break, aliases: "-b", type: :boolean, default: false)
      method_option(:skip, aliases: "-s", type: :array, default: [])
      method_option(:from, aliases: "-f", type: :string)
      method_option(:jobs_path, type: :string)
      method_option(:output_path, type: :string)

      # Worker options
      worker.params.each do |key, config|
        method_option(key, required: config[:required], type: config[:argument_type], default: config[:default])
      end

      define_method(identifier) do
        Dronejob::Base.jobs_path = options.jobs_path if options.jobs_path
        Dronejob::Base.output_path = options.output_path if options.output_path

        # Execute job
        worker.perform_now(options)
      end
    end

    desc :archive, "Archive Completed Jobs"
    method_option(:force, aliases: "-f", type: :boolean, default: false)
    def archive
      archive_dir = Workspace::Dir.new(File.join(Dir.pwd, "archive"))
      archive_dir.create
      Workspace::Dir.new(File.join(Dir.pwd, "tmp/jobs")).each_dir do |dir|
        dronejob_variables = dir.file("dronejob.yml").read_yaml
        next unless dronejob_variables[:dronejob_completed] == true

        Dronejob::Base.log("info", "Archiving #{dir.name}")
        archive_file = archive_dir.file("#{dir.name}.zip")
        if archive_file.exists? and !options.force
          Dronejob::Base.log("error", "Error: Archive '#{archive_file}' already exists. Use --force to replace.")
        else
          archive_file.delete if archive_file.exists?
          dir.compress(archive_file)
          dir.delete
        end
      end
    end

    desc :unarchive, "Unarchive Completed Job"
    archives_dir = Workspace::Dir.new(File.join(Dir.pwd, "archive"))
    archives = archives_dir.exists? ? archives_dir.children.map { |file| file.name.gsub(/\.zip$/, "") } : []
    method_option(:archive, type: :string, required: true, enum: archives)
    method_option(:force, aliases: "-f", type: :boolean, default: false)
    def unarchive
      archives_dir = Workspace::Dir.new(File.join(Dir.pwd, "archive"))
      archive_file = archives_dir.file("#{options.archive}.zip")
      Dronejob::Base.log("info", "Unarchiving '#{archive_file}'")
      target_dir = Workspace::Dir.new(File.join(Dir.pwd, "tmp/jobs")).dir(options.archive)
      if !options.force and target_dir.exists?
        Dronejob::Base.log("error", "Error: Job directory already exists")
      else
        target_dir.delete if target_dir.exists?
        archive_file.extract(target_dir)
      end
    end

    desc :continue, "Continue existing job"
    jobs_dir = Workspace::Dir.new(File.join(Dir.pwd, "tmp/jobs"))
    jobs = jobs_dir.exists? ? jobs_dir.children.map(&:name) : []
    method_option(:job, type: :string, required: true, enum: jobs)
    method_option(:from, aliases: "-f", type: :string)
    def continue
      job_dir = Workspace::Dir.new(File.join(Dir.pwd, "tmp/jobs")).dir(options.job)
      dronejob_config = job_dir.file("dronejob.yml").read_yaml
      job_params = dronejob_config[:job_params]
      job_identifier = dronejob_config[:job_identifier]
      worker = Dronejob::Loader.workers[job_identifier]
      job_params["from"] = options.from
      worker.perform_now(job_params)
    end
  end
end
