require "git"

module Dronejob
  module Modules
    module Git
      extend ActiveSupport::Concern

      included do
        attr_reader :git, :commits

        set_callback :job, :before, :git_init_job, if: :stateful?
        set_callback(:phase, :after, if: :stateful?) { git_commit(@phase) }
      end

      class_methods do
        def stateful(value = true)
          @stateful = value
        end

        def stateful?
          !!@stateful
        end
      end

      def stateful?
        self.class.stateful?
      end

      def git_init(path)
        @git = ::Git.init(path)
      end

      def git_commit(commit)
        unless @commits.keys.include?(commit.to_s)
          save_variables
          @git.add(all: true, force: true)
          @git.commit(commit.to_s, allow_empty: true)
          commit = @git.log(1).last
          @commits[commit] = commit.to_s
        end
      end

      def git_collect_commits
        @commits = {}
        begin
          @git.log(100).each do |c|
            @commits[c.message] = c.to_s
          end
        rescue StandardError
          # skip if log does not yet exist
        end
      end

      def git_clean
        @git.clean(force: true, d: true)
      end

      def git_reset(commit = nil)
        if commit
          @git.reset_hard(@commits[commit])
        else
          @git.reset_hard
        end
      end

      def git_init_job
        if self.class.stateful?
          load_variables
          git_init(@working_dir.to_s)
          git_collect_commits
          git_commit("start")
          if param(:from) == "start"
            info("starting from scratch")
            git_reset("start")
          elsif param(:from)
            info("starting from phase #{param(:from)}")
            phase = prev_phase(param(:from))
            raise("phase '#{param(:from)}' not found!") if phase.nil? or @commits[phase.to_s].nil?

            git_reset(phase.to_s)
          else
            git_reset
          end
          git_collect_commits
          git_clean
        end
      end
    end
  end
end
