module Dronejob
  module Modules
    module Workspace
      extend ActiveSupport::Concern

      included do
        cattr_accessor(:jobs_path) { "app/jobs" }
        cattr_accessor(:output_path) { "tmp/jobs" }
        set_callback :job, :after, :delete_working_dir
      end

      class_methods do
        def cleanup_on_complete(value = true)
          @cleanup_on_complete = value
        end

        def cleanup_on_complete?
          !!@cleanup_on_complete
        end

        def cleanup_on_error(value = true)
          @cleanup_on_error = value
        end

        def cleanup_on_error?
          !!@cleanup_on_error
        end
      end

      def file(path, &block)
        f = working_dir.file(path)
        unless block.nil?
          yield(f)
          f.write
        end
        f
      end

      def dir(path)
        working_dir.dir(path)
      end

      def working_dir
        unless @working_dir
          @working_dir = ::Workspace::Dir.new(File.join(Dir.pwd, self.class.output_path, uuid))
          @working_dir.create
        end
        @working_dir
      end

      def delete_working_dir
        working_dir.delete if self.class.cleanup_on_complete? && @working_dir
      end

      def asset_file(path)
        ::Workspace::Dir.new(File.join(Dir.pwd, "app/assets")).file(path)
      end

      def asset_dir(path)
        ::Workspace::Dir.new(File.join(Dir.pwd, "app/assets")).dir(path)
      end
    end
  end
end
